import traceback

import ipywidgets.widgets as widgets
from ipywidgets import Layout

from escape.core.objects import parameter_obj
from escape.core.common import iterate_parameters


class BaseParameterWidget(widgets.VBox):

    def __init__(self, paramobj, **kwargs):
        widgets.HBox.__init__(self, **kwargs)
        self.paramobj = paramobj
        self.children = self.init_children()

    def init_children(self):
        pass

    # Callback function
    def update(self, *args):
        pass
    
    def _repr_mimebundle_(self, **kwargs):
        self.update()
        return super(BaseParameterWidget, self)._repr_mimebundle_(**kwargs)


class IndependentParameterWidget(BaseParameterWidget):

    def init_children(self):
        paramobj = self.paramobj
        pmin, pmax = paramobj.get_unscaled_limits()
        pval = paramobj.get_unscaled_value()
        step = (pmax - pmin) / 100.0
        pval = paramobj.get_unscaled_value()
        self._disabled = False
        description = ""
        un = paramobj.units
        units = f"[{un}]" if un else ""
        if paramobj.scale != 1.0:
            description = '{0} x{1:.4g}{2}:'.format(paramobj.name,
                                   paramobj.scale,
                                   units)
        else:
            description = '{0}{1}:'.format(paramobj.name,
                                   units)
        self.description = description                           
        try:
            slider = widgets.FloatSlider(
                value=pval,
                min=pmin,
                max=pmax,
                step=step,
                readout=True,
                readout_format='.4g',
                continuous_update=False,
                description='',
                disabled=False,
                layout=Layout(width="90%", height="20px")
            )
            chkbox = widgets.Checkbox(
                value=not paramobj.fixed,
                description=description,
                description_tooltip="Fit/Fixed",
                indent=False,
                layout=Layout(width="90%", height="15px")
            )
            slider.observe(self.update_value, names='value')
            chkbox.observe(self.update_fixed, names='value')
            self.slider = slider
            self.chkbox = chkbox
            return [
                    chkbox,
                    slider,
                ]
        except:
            traceback.print_exc()
            return []
        
    def update(self, *args):
        self.update_value(*args)
        
    def update_value(self, *args):
        value = self.slider.get_interact_value()
        self.paramobj.value = value
        
    def pull_value(self):
        parobj = self.paramobj
        v = parobj.get_unscaled_value()
        self.slider.value = v
        self.chkbox.value = not parobj.fixed
        
    def update_fixed(self, *args):
        value = self.chkbox.get_interact_value()
        self.paramobj.fixed = not value
        
    @property
    def disabled(self):
        return self._disabled
    
    @disabled.setter
    def disabled(self, val):
        self._disabled = val
        for c in self.children:
            c.disabled = val
        

class DependentParameterWidget(widgets.VBox):
    
    label_tpl = "Result: {0:.4g}"
    
    def __init__(self, paramobj, **kwargs):
        widgets.VBox.__init__(self, _dom_classes=['widget-interact'])
        self.paramobj = paramobj
        self.ppn = ParametersPanel(paramobj)
        self.lbl = widgets.Label('')
        for par in self.ppn.children:
            par.slider.observe(self.update, names="value")
        self.children = [self.ppn, self.lbl]    
        
    # Callback function
    def update(self, *args):
        self.lbl.value = self.label_tpl.format(self.paramobj.value)


class ParametersPanel(widgets.VBox):
    
    def __init__(self, escape_obj, **kwargs):
        widgets.VBox.__init__(self, _dom_classes=['widget-interact'], **kwargs)
        self.escape_obj = escape_obj
        numpars = self.escape_obj.num_of_params
        c = [ IndependentParameterWidget(self.escape_obj.parameter(i),
                                         layout=Layout(max_height="50px",
                                                       min_height="50px",
                                                       overflow_x='hidden',
                                                       overflow_y='hidden')
                                                       ) 
                               for i in range(numpars)]
        self.children = c
        self._disabled = False
    
    @property
    def disabled(self):
        return self._disabled
    
    @disabled.setter
    def disabled(self, val):
        self._disabled = val
        for c in self.children:
            c.disabled = val

